@@ -52,11 +52,12 @@ if __name__ == "__main__":
             space,
             parallel_strategy='async',
             batch_size=n_workers,
-            batch_strategy='median_imputation',
+            batch_strategy='default',
             num_objs=1,
             num_constraints=0,
             max_runs=50,
-            surrogate_type='gp',
+            # surrogate_type='gp',
+            surrogate_type='auto',
             time_limit_per_trial=180,
             task_id='distributed_opt',
             ip="",
