@@ -22,9 +22,9 @@ class mqSMBO(BOBase):
                  sample_strategy: str = 'bo',
                  max_runs=200,
                  time_limit_per_trial=180,
-                 surrogate_type=None,
-                 acq_type=None,
-                 acq_optimizer_type='local_random',
+                 surrogate_type='auto',
+                 acq_type='auto',
+                 acq_optimizer_type='auto',
                  initial_runs=3,
                  init_strategy='random_explore_first',
                  initial_configurations=None,
@@ -32,7 +32,8 @@ class mqSMBO(BOBase):
                  history_bo_data: List[OrderedDict] = None,
                  logging_dir='logs',
                  task_id='default_task_id',
-                 random_state=1,
+                 random_state=None,
+                 advisor_kwargs: dict = None,
                  ip="",
                  port=13579,
                  authkey=b'abc',):
@@ -40,14 +41,24 @@ class mqSMBO(BOBase):
         if task_id is None:
             raise ValueError('Task id is not SPECIFIED. Please input task id first.')
 
-        self.task_info = {'num_constraints': num_constraints, 'num_objs': num_objs}
+        self.num_objs = num_objs
+        self.num_constraints = num_constraints
         self.FAILED_PERF = [MAXINT] * num_objs
         super().__init__(objective_function, config_space, task_id=task_id, output_dir=logging_dir,
                          random_state=random_state, initial_runs=initial_runs, max_runs=max_runs,
                          sample_strategy=sample_strategy, time_limit_per_trial=time_limit_per_trial,
                          history_bo_data=history_bo_data)
+
+        self.parallel_strategy = parallel_strategy
+        self.batch_size = batch_size
+        max_queue_len = max(100, 3 * batch_size)
+        self.master_messager = MasterMessager(ip, port, authkey, max_queue_len, max_queue_len)
+
+        advisor_kwargs = advisor_kwargs or {}
         if parallel_strategy == 'sync':
-            self.config_advisor = SyncBatchAdvisor(config_space, self.task_info,
+            self.config_advisor = SyncBatchAdvisor(config_space,
+                                                   num_objs=num_objs,
+                                                   num_constraints=num_constraints,
                                                    batch_size=batch_size,
                                                    batch_strategy=batch_strategy,
                                                    initial_trials=initial_runs,
@@ -61,9 +72,12 @@ class mqSMBO(BOBase):
                                                    ref_point=ref_point,
                                                    task_id=task_id,
                                                    output_dir=logging_dir,
-                                                   random_state=random_state)
+                                                   random_state=random_state,
+                                                   **advisor_kwargs)
         elif parallel_strategy == 'async':
-            self.config_advisor = AsyncBatchAdvisor(config_space, self.task_info,
+            self.config_advisor = AsyncBatchAdvisor(config_space,
+                                                    num_objs=num_objs,
+                                                    num_constraints=num_constraints,
                                                     batch_size=batch_size,
                                                     batch_strategy=batch_strategy,
                                                     initial_trials=initial_runs,
@@ -77,15 +91,11 @@ class mqSMBO(BOBase):
                                                     ref_point=ref_point,
                                                     task_id=task_id,
                                                     output_dir=logging_dir,
-                                                    random_state=random_state)
+                                                    random_state=random_state,
+                                                    **advisor_kwargs)
         else:
             raise ValueError('Invalid parallel strategy - %s.' % parallel_strategy)
 
-        self.parallel_strategy = parallel_strategy
-        self.batch_size = batch_size
-        max_queue_len = max(100, 3 * batch_size)
-        self.master_messager = MasterMessager(ip, port, authkey, max_queue_len, max_queue_len)
-
     def async_run(self):
         config_num = 0
         result_num = 0
@@ -108,9 +118,11 @@ class mqSMBO(BOBase):
                     break
                 # Report result.
                 result_num += 1
-                config, trial_state, constraints, objs, elapsed_time = observation
-                if objs is None:
-                    observation = Observation(config, trial_state, constraints, self.FAILED_PERF, elapsed_time)
+                if observation.objs is None:
+                    observation = Observation(
+                        config=observation.config, objs=self.FAILED_PERF, constraints=observation.constraints,
+                        trial_state=observation.trial_state, elapsed_time=observation.elapsed_time,
+                    )
                 self.config_advisor.update_observation(observation)
                 self.logger.info('Master: Get %d observation: %s' % (result_num, str(observation)))
 
@@ -138,9 +150,11 @@ class mqSMBO(BOBase):
                     continue
                 # Report result.
                 result_num += 1
-                config, trial_state, constraints, objs, elapsed_time = observation
-                if objs is None:
-                    observation = Observation(config, trial_state, constraints, self.FAILED_PERF, elapsed_time)
+                if observation.objs is None:
+                    observation = Observation(
+                        config=observation.config, objs=self.FAILED_PERF, constraints=observation.constraints,
+                        trial_state=observation.trial_state, elapsed_time=observation.elapsed_time,
+                    )
                 self.config_advisor.update_observation(observation)
                 self.logger.info('Master: In the %d-th batch [%d], observation is: %s'
                                  % (batch_id, result_num, str(observation)))
