@@ -6,13 +6,15 @@ import json
 import collections
 from typing import List, Union
 import numpy as np
+from ConfigSpace import CategoricalHyperparameter, OrdinalHyperparameter, Constant
 from openbox.utils.constants import MAXINT, SUCCESS
 from openbox.utils.config_space import Configuration, ConfigurationSpace
 from openbox.utils.logging_utils import get_logger
 from openbox.utils.multi_objective import Hypervolume, get_pareto_front
-from openbox.utils.config_space.space_utils import get_config_from_dict
+from openbox.utils.config_space.space_utils import get_config_from_dict, get_config_values
 from openbox.utils.visualization.plot_convergence import plot_convergence
 from openbox.core.base import Observation
+from openbox.utils.transform import get_transform_function
 
 Perf = collections.namedtuple(
     'perf', ['cost', 'time', 'status', 'additional_info'])
@@ -62,8 +64,8 @@ class HistoryContainer(object):
             self.perfs.append(objs[0])
         else:
             self.perfs.append(objs)
-        self.trial_states.append(trial_state)
         self.constraint_perfs.append(constraints)  # None if no constraint
+        self.trial_states.append(trial_state)
         self.elapsed_times.append(elapsed_time)
 
         transform_perf = False
@@ -126,22 +128,17 @@ class HistoryContainer(object):
             self.incumbent_value = perf
             self.incumbents.append((config, perf))
 
-    def get_transformed_perfs(self):
+    def get_transformed_perfs(self, transform=None):
+        # set perf of failed trials to current max
         transformed_perfs = self.perfs.copy()
         for i in self.transform_perf_index:
             transformed_perfs[i] = self.max_y
+
         transformed_perfs = np.array(transformed_perfs, dtype=np.float64)
+        transformed_perfs = get_transform_function(transform)(transformed_perfs)
         return transformed_perfs
 
-    def get_transformed_constraint_perfs(self, bilog_transform=True):
-        def bilog(y: np.ndarray):
-            """Magnify the difference between y and 0"""
-            idx = (y >= 0)
-            y_copy = y.copy()
-            y_copy[idx] = np.log(1 + y_copy[idx])
-            y_copy[~idx] = -np.log(1 - y_copy[~idx])
-            return y_copy
-
+    def get_transformed_constraint_perfs(self, transform='bilog'):
         if self.num_constraints == 0:
             return None
 
@@ -152,8 +149,7 @@ class HistoryContainer(object):
             transformed_constraint_perfs[i] = max_c
 
         transformed_constraint_perfs = np.array(transformed_constraint_perfs, dtype=np.float64)
-        if bilog_transform:
-            transformed_constraint_perfs = bilog(transformed_constraint_perfs)
+        transformed_constraint_perfs = get_transform_function(transform)(transformed_constraint_perfs)
         return transformed_constraint_perfs
 
     def get_perf(self, config: Configuration):
@@ -290,91 +286,89 @@ class HistoryContainer(object):
         hip.Experiment.from_iterable(visualize_data).display()
         return
 
-    def get_importance(self, config_space=None, return_list=False):
+    def get_importance(self, method='fanova', config_space=None, return_dict=False):
         def _get_X(configurations, config_space):
-            from ConfigSpace import CategoricalHyperparameter, OrdinalHyperparameter, Constant
-            X_from_dict = np.array([list(config.get_dictionary().values()) for config in configurations])
+            X_from_dict = np.array([get_config_values(config, config_space) for config in configurations],
+                                   dtype=object)
             X_from_array = np.array([config.get_array() for config in configurations])
             discrete_types = (CategoricalHyperparameter, OrdinalHyperparameter, Constant)
             discrete_idx = [isinstance(hp, discrete_types) for hp in config_space.get_hyperparameters()]
             X = X_from_dict.copy()
             X[:, discrete_idx] = X_from_array[:, discrete_idx]
+            X = X.astype(X_from_array.dtype)
             return X
 
-        try:
-            import pyrfr.regression as reg
-            import pyrfr.util
-        except ModuleNotFoundError:
-            self.logger.error(
-                'To use get_importance(), please install pyrfr: '
-                'https://open-box.readthedocs.io/en/latest/installation/install_pyrfr.html'
-            )
-            raise
-        from openbox.utils.fanova import fANOVA
-        from terminaltables import AsciiTable
-
+        Y = np.array(self.get_transformed_perfs(transform=None))
+        if len(Y.shape) == 1:
+            Y = np.reshape(Y, (len(Y), 1))
+        num_objs = Y.shape[1]
         if config_space is None:
             config_space = self.config_space
         if config_space is None:
             raise ValueError('Please provide config_space to show parameter importance!')
-
-        X = _get_X(self.configurations, config_space)
-        Y = np.array(self.get_transformed_perfs())
-
-        # create an instance of fanova with data for the random forest and the configSpace
-        f = fANOVA(X=X, Y=Y, config_space=config_space)
-
-        # marginal for first parameter
         keys = [hp.name for hp in config_space.get_hyperparameters()]
-        importance_list = list()
-        for key in keys:
-            p_list = (key,)
-            res = f.quantify_importance(p_list)
-            individual_importance = res[(key,)]['individual importance']
-            importance_list.append([key, individual_importance])
-        importance_list.sort(key=lambda x: x[1], reverse=True)
+        importance_dict = {key: [] for key in keys}
 
-        if return_list:
-            return importance_list
+        if method == 'shap':
+            import shap
+            from lightgbm import LGBMRegressor
+            from terminaltables import AsciiTable
 
-        for item in importance_list:
-            item[1] = '%.6f' % item[1]
-        table_data = [["Parameters", "Importance"]] + importance_list
-        importance_table = AsciiTable(table_data).table
-        return importance_table
+            for hp in config_space.get_hyperparameters():
+                if isinstance(hp, CategoricalHyperparameter):
+                    print("SHAP can not support categorical hyperparameters well. To analyze a space with categorical "
+                          "hyperparameters, we recommend setting the method to fanova.")
 
-    def get_shap_importance(self, config_space=None, return_list=False):
-        import shap
-        from lightgbm import LGBMRegressor
-        from terminaltables import AsciiTable
+            X = _get_X(self.configurations, config_space)
 
-        if config_space is None:
-            config_space = self.config_space
-        if config_space is None:
-            raise ValueError('Please provide config_space to show parameter importance!')
+            for col_idx in range(num_objs):
+                # Fit a LightGBMRegressor with observations
+                lgbr = LGBMRegressor()
+                lgbr.fit(X, Y[:, col_idx])
+                explainer = shap.TreeExplainer(lgbr)
+                shap_values = explainer.shap_values(X)
+                feature_importance = np.mean(np.abs(shap_values), axis=0)
 
-        X = np.array([list(config.get_dictionary().values()) for config in self.configurations])
-        Y = np.array(self.get_transformed_perfs())
+                keys = [hp.name for hp in config_space.get_hyperparameters()]
+                for i, hp_name in enumerate(keys):
+                    importance_dict[hp_name].append(feature_importance[i])
 
-        # Fit a LightGBMRegressor with observations
-        lgbr = LGBMRegressor()
-        lgbr.fit(X, Y)
-        explainer = shap.TreeExplainer(lgbr)
-        shap_values = explainer.shap_values(X)
-        feature_importance = np.mean(np.abs(shap_values), axis=0)
+        elif method == 'fanova':
+            try:
+                import pyrfr.regression as reg
+                import pyrfr.util
+            except ModuleNotFoundError:
+                self.logger.error(
+                    'To use get_importance(), please install pyrfr: '
+                    'https://open-box.readthedocs.io/en/latest/installation/install_pyrfr.html'
+                )
+                raise
+            from openbox.utils.fanova import fANOVA
+            from terminaltables import AsciiTable
+
+            X = _get_X(self.configurations, config_space)
+
+            for col_idx in range(num_objs):
+                # create an instance of fanova with data for the random forest and the configSpace
+                f = fANOVA(X=X, Y=Y[:, col_idx], config_space=config_space)
+
+                # marginal for first parameter
+                for key in keys:
+                    p_list = (key,)
+                    res = f.quantify_importance(p_list)
+                    individual_importance = res[(key,)]['individual importance']
+                    importance_dict[key].append(individual_importance)
+        else:
+            raise ValueError("Invalid method for feature importance. Valid choices: 'shap' or 'fanova'.")
+
+        if return_dict:
+            return importance_dict
 
-        keys = [hp.name for hp in config_space.get_hyperparameters()]
         importance_list = []
-        for i, hp_name in enumerate(keys):
-            importance_list.append([hp_name, feature_importance[i]])
-        importance_list.sort(key=lambda x: x[1], reverse=True)
-
-        if return_list:
-            return importance_list
-
-        for item in importance_list:
-            item[1] = '%.6f' % item[1]
-        table_data = [["Parameters", "Importance"]] + importance_list
+        for key, value in importance_dict.items():
+            importance_list.append([key] + ['%.6f' % imp for imp in value])
+        table_head = [["Parameters"] + ["Obj%d Importance" % i for i in range(1, num_objs + 1)]]
+        table_data = table_head + importance_list
         importance_table = AsciiTable(table_data).table
         return importance_table
 
@@ -387,36 +381,70 @@ class HistoryContainer(object):
         fn : str
             file name
         """
-        data = [(k.get_dictionary(), float(v)) for k, v in self.data.items()]  # todo: all configs
 
-        with open(fn, "w") as fp:
-            json.dump({"data": data}, fp, indent=2)
+        data = []
+        for idx, (config, perf, constraint_perf, trial_state, elapsed_time) in enumerate(zip(
+                self.configurations, self.perfs, self.constraint_perfs, self.trial_states, self.elapsed_times,
+        )):
+            config_dict = config.get_dictionary()
+            _perf = [float(p) for p in perf] if self.num_objs > 1 else float(perf)
+            _constraint_perf = [float(c) for c in constraint_perf] if self.num_constraints > 0 else constraint_perf
 
-    def load_history_from_json(self, cs: ConfigurationSpace, fn: str = "history_container.json"):  # todo: all configs
-        """Load and runhistory in json representation from disk.
+            data_item = dict(
+                index=idx,
+                config=config_dict,
+                perf=_perf,
+                constraint_perf=_constraint_perf,
+                trial_state=trial_state,
+                elapsed_time=elapsed_time,
+            )
+            data.append(data_item)
+
+        with open(fn, 'w') as fp:
+            json.dump({'data': data}, fp, indent=2)
+        self.logger.info('Save history to %s' % fn)
+
+    def load_history_from_json(self, fn: str = "history_container.json", config_space: ConfigurationSpace = None):
+        """Load history in json representation from disk.
         Parameters
         ----------
         fn : str
             file name to load from
-        cs : ConfigSpace
+        config_space : ConfigSpace
             instance of configuration space
         """
+
+        if config_space is None:
+            config_space = self.config_space
+        if config_space is None:
+            raise ValueError('Please provide config_space to load_history_from_json!')
+
         try:
-            with open(fn) as fp:
+            with open(fn, 'r') as fp:
                 all_data = json.load(fp)
         except Exception as e:
             self.logger.warning(
-                'Encountered exception %s while reading runhistory from %s. '
+                'Encountered exception %s while reading history from %s. '
                 'Not adding any runs!', e, fn,
             )
             return
-        _history_data = collections.OrderedDict()
-        # important to use add method to use all data structure correctly
-        for k, v in all_data["data"]:
-            config = get_config_from_dict(k, cs)
-            perf = float(v)
-            _history_data[config] = perf
-        return _history_data
+
+        for data_item in all_data['data']:
+            config_dict = data_item['config']
+            perf = data_item['perf']
+            constraint_perf = data_item['constraint_perf']
+            trial_state = data_item['trial_state']
+            elapsed_time = data_item['elapsed_time']
+
+            config = get_config_from_dict(config_dict, config_space)
+            objs = perf if self.num_objs > 1 else [perf]
+
+            observation = Observation(
+                config=config, objs=objs, constraints=constraint_perf, trial_state=trial_state,
+                elapsed_time=elapsed_time)
+            self.update_observation(observation)
+
+        self.logger.info('Load history from %s. len = %d.' % (fn, len(all_data['data'])))
 
 
 class MOHistoryContainer(HistoryContainer):
@@ -424,8 +452,8 @@ class MOHistoryContainer(HistoryContainer):
     Multi-Objective History Container
     """
 
-    def __init__(self, task_id, num_objs, num_constraints=0, ref_point=None):
-        super().__init__(task_id=task_id, num_constraints=num_constraints)
+    def __init__(self, task_id, num_objs, num_constraints=0, config_space=None, ref_point=None):
+        super().__init__(task_id=task_id, num_constraints=num_constraints, config_space=config_space)
         self.pareto = collections.OrderedDict()
         self.num_objs = num_objs
         self.mo_incumbent_value = [MAXINT] * self.num_objs
@@ -522,20 +550,22 @@ class MultiStartHistoryContainer(object):
     History container for multistart algorithms.
     """
 
-    def __init__(self, task_id, num_objs=1, num_constraints=0, ref_point=None):
+    def __init__(self, task_id, num_objs=1, num_constraints=0, config_space=None, ref_point=None):
         self.task_id = task_id
         self.num_objs = num_objs
         self.num_constraints = num_constraints
         self.history_containers = []
+        self.config_space = config_space
         self.ref_point = ref_point
         self.current = None
         self.restart()
 
     def restart(self):
         if self.num_objs == 1:
-            self.current = HistoryContainer(self.task_id, self.num_constraints)
+            self.current = HistoryContainer(self.task_id, self.num_constraints, self.config_space)
         else:
-            self.current = MOHistoryContainer(self.task_id, self.num_objs, self.num_constraints, self.ref_point)
+            self.current = MOHistoryContainer(
+                self.task_id, self.num_objs, self.num_constraints, self.config_space, self.ref_point)
         self.history_containers.append(self.current)
 
     def get_configs_for_all_restarts(self):
@@ -591,11 +621,11 @@ class MultiStartHistoryContainer(object):
     def successful_perfs(self):
         return self.current.successful_perfs
 
-    def get_transformed_perfs(self):
-        return self.current.get_transformed_perfs
+    def get_transformed_perfs(self, *args, **kwargs):
+        return self.current.get_transformed_perfs(*args, **kwargs)
 
-    def get_transformed_constraint_perfs(self):
-        return self.current.get_transformed_constraint_perfs
+    def get_transformed_constraint_perfs(self, *args, **kwargs):
+        return self.current.get_transformed_constraint_perfs(*args, **kwargs)
 
     def get_perf(self, config: Configuration):
         for history_container in self.history_containers:
@@ -646,13 +676,13 @@ class MultiStartHistoryContainer(object):
         """
         self.current.save_json(fn)
 
-    def load_history_from_json(self, cs: ConfigurationSpace, fn: str = "history_container.json"):
-        """Load and runhistory in json representation from disk.
+    def load_history_from_json(self, fn: str = "history_container.json", config_space: ConfigurationSpace = None):
+        """Load history in json representation from disk.
         Parameters
         ----------
         fn : str
             file name to load from
-        cs : ConfigSpace
+        config_space : ConfigSpace
             instance of configuration space
         """
-        self.current.load_history_from_json(cs, fn)
+        self.current.load_history_from_json(fn, config_space)
